program cap_hdus;

{$APPTYPE CONSOLE}

// ----------------------------------------------
//   cap_hdus
//   MobileHackerz : http://mobilehackerz.jp
// ----------------------------------------------
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  This Program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation; either version 2, or (at your option)      *
 *  any later version.                                                       *
 *                                                                           *
 *  This Program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with GNU Make; see the file COPYING.  If not, write to             *
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.    *
 *  http://www.gnu.org/copyleft/gpl.html                                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

uses
  SysUtils,
  DateUtils,
  ActiveX,
  Classes,
  Windows,
  pDriver,
  B25D_delphi in 'B25D_delphi.pas';

const
  VERSION      = '0.07';
  MUTEXNAME    = 'cap_hdus_mutex';
  MUTEXTIMEOUT = 30;
  B25RETRY     = 3;
  B25RETRYWAIT = 50;
  DEFAULTSEC   = 20;

type
  TIOInterface = class(TObject)
  private
    { Private 錾 }
    FTS      :TFileStream;
    FSave    :Boolean;
    FVerbose :Boolean;
    FB25Dec  :TB25Decoder;
  public
    { Public 錾 }
    constructor Create;
    procedure ReceiveTSStream(HDUS:TObject; ABuffer: PByte; ASize: Integer);
    procedure LogEvent(HDUS:TObject; AText: String);
    function CreateFile(FileName: String):Boolean;
    property Save       :Boolean     read FSave    write FSave;
    property Verbose    :Boolean     read FVerbose write FVerbose;
    property B25Decoder :TB25Decoder read FB25Dec  write FB25Dec;
  end;


// --------------------------------------------------------
//   TIOInterface
// --------------------------------------------------------

constructor TIOInterface.Create;
begin
  inherited;
  FSave    := False;
  FVerbose := False;
  FTS      := nil;
end;

function TIOInterface.CreateFile(FileName: String):Boolean;
begin
  Result := True;
  try
    if Assigned(FTS) then FreeAndNil(FTS);
    // FileStream̎dlŁAfmCreate̎SharedŊJȂ̂
    // xCreateĂReadWriteŊJȂ
    FTS := TFileStream.Create(FileName,fmCreate or fmShareDenyNone);
    FTS.Free;
    FTS := TFileStream.Create(FileName,fmOpenReadWrite or fmShareDenyNone);
  except
    Result := False;
  end;
end;

procedure TIOInterface.ReceiveTSStream(HDUS:TObject; ABuffer: PByte; ASize: Integer);
var
  DecBuffer :PByte;
  DecSize   :DWORD;
  Decoded   :Boolean;
  i         :Integer;
  flg       :Boolean;
begin
  if Assigned(FTS) then
  begin
    if (FSave) then
    begin
      Decoded := False;
      if Assigned(FB25Dec) then
      begin
        if (FB25Dec.DLLReady) and (FB25Dec.Initialized) then
        begin
          i := 0;
          repeat
          begin
            flg := FB25Dec.Decode(ABuffer,ASize,DecBuffer,DecSize);
            Inc(i);
            if (not flg) then Sleep(B25RETRYWAIT);
          end until ((i>=B25RETRY) or flg);
          Decoded := flg;
        end;
      end;
      //
      if (Decoded) then
      begin
        if (DecSize>0) then FTS.Write(DecBuffer^,DecSize);
      end
        else
      begin
        if (ASize>0)   then FTS.Write(ABuffer^,ASize);
      end;
    end;
  end;
end;

procedure TIOInterface.LogEvent(HDUS:TObject; AText: String);
begin
  if (FVerbose) then Writeln(AText);
end;


// ----------------------------
//   ɕϊ
// ----------------------------

function StrToTime3(str:String; var ret:TDateTime):Boolean;
var
  st: array[0..2] of String;
  i,p: Integer;
  ch: Char;
  h,m,s,d: Integer;
begin
  str := Trim(str);
  i := 0;
  p := 0;
  st[0] := ''; st[1] := ''; st[2] := '';
  while ((i<Length(str)) and (p<=2)) do
  begin
    ch := str[i+1];
    i := i + 1;
    if ((ord(ch) >= $30) and (ord(ch) <= $39)) then
    begin
      st[p] := st[p]+ch;
    end
      else
    begin
      if (ch = ':') then
      begin
        if (st[p]<>'') then
        begin
          p := p + 1;
        end;
      end
        else
      begin
        Result := False;
        exit;
      end;
    end;
  end;
  h := StrToIntDef(st[0],-1);
  m := StrToIntDef(st[1],-1);
  s := StrToIntDef(st[2],0);
  if ((h<0) or (m<0)) then
  begin
    Result := False;
  end
    else
  begin
    d := h div 24;
    h := h mod 24;
    ret := EncodeTime(h,m,s,0) + d;
    Result := True;
  end;
end;



// --------------------------------------------------------
//   C
// --------------------------------------------------------

var
  hMutex     :THandle;
  i,prmnum,p :Integer;
  prm,prmcmd :String;
  prm1,prm2  :String;
  flg        :Boolean;
  //
  FileName   :String;
  Channel    :Integer;
  ChName     :String;
  StartTime  :TDateTime;
  WakeTime   :TDateTime;
  EndTime    :TDateTime;
  NowTime    :TDateTime;
  dt         :TDateTime;
  TunerSel   :String;
  TunerID    :String;
  DriverPath :String;
  HDUSDLL    :TDriverDLL;
  HDUS       :Pointer;
  HDUS_IO    :TIOInterface;
  Verbose    :Boolean;
  B25Dec     :TB25Decoder;
begin
	writeln('cap_hdus : R}hCTSL^c[ for MonsterTV HDUS Ver'+VERSION);
  writeln('by MobileHackerz (http://mobilehackerz.jp)');
  writeln('');
  writeln('Special Thanks to YN_JP40___9 UNFyHWN3Jg');
  writeln('');

  // ftHgp[^
  NowTime    := Now;
  FileName   := 'test.ts2';
  Channel    := -1;
  ChName     := 'ݒ';
  StartTime  := NowTime;
  EndTime    := 0;
  TunerSel   := '';
  TunerID    := '';
  Verbose    := False;
  DriverPath := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));

  // Usage
  if (ParamCount = 0) then
  begin
  	writeln('g');
    writeln('cap_hdus [t@C] [@`l] [] [Il]');
    writeln('  t@C  Fgqȗ".ts2"Bt@Cȗ"test.ts2"B');
    writeln('  `l  FnfW"@13"`"@62"BCATVgϊ"@C36"`"@C63"B');
    writeln('          Fl"30"ŋL^()B"19:00-19:30"ŎwB');
    writeln('  "`[iID"FΏۂMonsterTV HDUSIDwBȗ͋󂫃`[i[̂ǂꂩB');
    writeln('');
    writeln('  -v          FfobObZ[W\');
    writeln('gp၄');
    writeln('ucap_hdus j[X @27 30v 27ch"j[X.ts2"ō쐬BL^Ԃ30B');
    writeln('ucap_hdus 19:00-19:30 @27v 27ch 19:00-19:30"test.ts2"ō쐬B');
    ExitCode := -1;
    exit;
  end;

  // R}hC`FbN
  for i:=1 to ParamCount do
  begin
    prm := ParamStr(i);
    prmcmd := UpperCase(Copy(prm,1,1));
    prmnum := StrToIntDef(Copy(prm,2,Length(prm)),-1);
    // ԋLq
    prm1 := '';
    prm2 := '';
    p := Pos('-',prm);
    if (p>0) then
    begin
      prm1 := Copy(prm,1,p-1);
      prm2 := Copy(prm,p+1,Length(prm));
      if ((not StrToTime3(prm1,dt)) or (not StrToTime3(prm2,dt))) then
      begin
        prm1 := '';
        prm2 := '';
      end;
    end;
    if (prm1='') then
    begin
      if (StrToTime3(prm,dt)) then prm1 := prm;
    end;
    //
    if (prmcmd='@') then
    begin
      // Channel
      flg := False;
      prmcmd := Copy(prm,2,1);
      if (prmcmd='C') then
      begin
        prmnum := StrToIntDef(Copy(prm,3,Length(prm)),-1);
        if (prmnum>=36) then
        begin
          // Cable UHF
          Channel := 303000 + (prmnum-36)*6000;
          ChName  := 'CATV'+IntToStr(prmnum)+'ch';
          flg := True;
        end;
      end
        else
      begin
        if (prmnum>=13) then
        begin
          // UHF
          Channel := 473000 + (prmnum-13)*6000;
          ChName  := 'UHF'+IntToStr(prmnum)+'ch';
          flg := True;
        end;
      end;
      if (not flg) then
      begin
        if (Copy(LowerCase(prm),1,8)='@device:') then
        begin
          // `[iID
          TunerSel := prm;
        end
          else
        begin
          writeln('`lݒ肪ُł');
          ExitCode := -1;
          exit;
        end;
      end;
    end
      else if (StrToIntDef(prm,-1)>0) then
    begin
      // Time (Remain)
      EndTime := IncMinute(StartTime,StrToIntDef(prm,0));
    end
      else if (prm1<>'') then
    begin
      // TIme (StartTime)
      if (StrToTime3(prm1,dt)) then
      begin
        if ((DateOf(NowTime)+dt)<NowTime) then
        begin
          // 
          StartTime := IncDay(DateOf(NowTime),1)+dt;
        end
          else
        begin
          // 
          StartTime := DateOf(NowTime)+dt;
        end;
      end;
      // Time (EndTime)
      if (prm2<>'') then
      begin
        // TIme (EndTime)
        if (StrToTime3(prm2,dt)) then
        begin
          if ((DateOf(NowTime)+dt)<NowTime) then
          begin
            // 
            EndTime := IncDay(DateOf(NowTime),1)+dt;
          end
            else
          begin
            // 
            EndTime := DateOf(NowTime)+dt;
          end;
        end;
        if (StartTime>EndTime) then EndTime := IncDay(EndTime,1);
      end;
    end
      else if (UpperCase(prm)='-V') then
    begin
      // Verbose[h
      Verbose := True;
    end
      else if (UpperCase(prm)='-X') then
    begin
      // HDUS{hCoΉ[h
      // ʂɂ̂ŃIvV폜(G[ɂȂ悤肾c)
    end
      else
    begin
      // t@C
      FileName := prm;
      if (ExtractFileExt(FileName)='') then FileName := ChangeFileExt(FileName,'.ts2');
    end;
  end;

  // ftHgI(20b)
  if (EndTime=0) then EndTime := IncSecond(StartTime,DEFAULTSEC);

  // N(10bO)
  WakeTime := IncSecond(StartTime,-10);

  // `l`FbN
  if (Channel<0) then
  begin
    writeln('`l̐ݒ肪Kvł');
    ExitCode := -1;
    exit;
  end;

  // B25Decoder_dIuWFNg
  B25Dec := TB25Decoder.Create;

  // HDUSIuWFNgo
  HDUS_IO := TIOInterface.Create;
  HDUS_IO.Verbose := Verbose;
  HDUS_IO.CreateFile(FileName);
  HDUS_IO.B25Decoder := B25Dec;

  // ݒmF
  writeln('----------------------------');
  writeln('ݎ@@F'+DateTimeToStr(NowTime));
  writeln('L^Jn@@F'+DateTimeToStr(StartTime));
  writeln('L^I@@F'+DateTimeToStr(EndTime));
  writeln('`l@F'+ChName+' ('+IntToStr(Channel)+'kHz)');
  writeln('t@C@F'+FileName);
  writeln('----------------------------');

  // N҂
  if (WakeTime>Now) then
  begin
    writeln('Jn҂Ă܂...');
    while(WakeTime>Now) do Sleep(1000);
  end;

  // HDUSIuWFNg
  CoInitialize(nil);
  hMutex  := 0;
  HDUSDLL := nil;
  HDUS    := nil;
  try
    // Mutexirj
    hMutex :=CreateMutex(nil, False, MUTEXNAME);
    if WaitForSingleObject(hMutex, MUTEXTIMEOUT*1000) = WAIT_TIMEOUT then
    begin
      raise Exception.Create('cap_hdus䒆œł܂ł');
    end;

    HDUSDLL := TDriverDLL.Create(DriverPath+'SegDriver_hdus.dll');
    HDUS    := HDUSDLL.MakeDriverInstance;
    HDUSDLL.SetLogCallback(HDUS,HDUS_IO.LogEvent);
    HDUSDLL.SetReceiveTSCallback(HDUS,HDUS_IO.ReceiveTSStream);

    // `[iꗗ\
    for i:=0 to HDUSDLL.Count-1 do
    begin
      if (((TunerSel='') and (i=0)) or (TunerSel=HDUSDLL.TunerID[i])) then
      begin
        writeln('['+IntToStr(i)+']:'+HDUSDLL.TunerName[i]);
        if (Verbose) then writeln(HDUSDLL.TunerID[i]);
        TunerID := HDUSDLL.TunerID[i];
      end
        else
      begin
        writeln(' '+IntToStr(i)+' :'+HDUSDLL.TunerName[i]);
        if (Verbose) then writeln(HDUSDLL.TunerID[i]);
      end;
    end;
    if (TunerID='') then raise Exception.Create('`[i܂');

    writeln('`[iID  F'+TunerID);
    writeln('----------------------------');

    // t@C쐬
    if (not HDUS_IO.CreateFile(FileName)) then raise Exception.Create('t@Cł܂');

    // N
    writeln('y`[i>'+DateTimeToStr(Now)+'z');
    i := 0;
    repeat
    begin
      flg := B25Dec.Initialize(4);
      Inc(i);
      if (not flg) then Sleep(B25RETRYWAIT);
    end until ((i>=B25RETRY) or flg);

    if (not HDUSDLL.Activate(HDUS,PChar(TunerID))) then raise Exception.Create('`[iANeBx[gł܂');
    if (not HDUSDLL.Tune(HDUS,Channel))            then raise Exception.Create('gݒł܂');

    // Mutexi܂Ŕrj
    ReleaseMutex(hMutex);
    hMutex := 0;

    // Jn҂
    while(StartTime>Now) do Sleep(100);

    // Jn
    writeln('yL^Jn>'+DateTimeToStr(Now)+'z');
    HDUS_IO.Save := True;

    // I҂
    while(EndTime>Now) do Sleep(1000);

    // I
    writeln('yL^I>'+DateTimeToStr(Now)+'z');
    HDUS_IO.Save := False;
  except
    on E:Exception do
    begin
      writeln(E.Message);
      ExitCode := -1;

      if Assigned(HDUS)    then HDUSDLL.Deactivate(HDUS);
      if Assigned(HDUS_IO) then FreeAndNil(HDUS_IO);
      CoUninitialize;
      if Assigned(B25Dec)  then FreeAndNil(B25Dec);
      if (hMutex<>0)       then ReleaseMutex(hMutex);
      exit;
    end;
  end;

  // Exit
  ExitCode := 0;

  if Assigned(HDUS)    then HDUSDLL.Deactivate(HDUS);
  if Assigned(HDUS_IO) then FreeAndNil(HDUS_IO);
  CoUninitialize;
  if Assigned(B25Dec)  then FreeAndNil(B25Dec);
end.
